/*
 *  C wrapper routines for ScaLapack
 *  Time-stamp: <31 Oct 2002 21:23 GMT cscalapack.c maising> 
 *  HISTORY
 *     23-11-2001   KSM    original version
 */

/*
 * please read the comments in cscalapack.h
 */
#include "cscalapack.h"

#if defined(AIX)

/*
 * NOTE: if haven't checked whether pdtrtrs and pdtrsm are really 
 * equivalent - this may not work!
 */
void cpdtrtrs(char *UPLO, char *TRANS, char *DIAG, int N, int NRHS, 
	      double *A, int IA, int JA, int *DESCA,
	      double *B, int IB, int JB, int *DESCB, int *INFO)
{
  static char side = 'L';

  pdtrsm(&side, UPLO, TRANS, DIAG, N, NRHS, 1., 
	 A, IA, JA, DESCA, B, IB, JB, DESCB);
  return;
}

int cpdelset(double *A, int IA, int JA, int *DESCA, double ALPHA)
{
  return 
    pdelset(A, &IA, &JA, DESCA, &ALPHA);
}

int cpdelget(char *SCOPE, char *TOP, double *ALPHA, 
	     double *A, int IA, int JA, int *DESCA)
{
  return
    pdelget(SCOPE, TOP, ALPHA, A, &IA, &JA, DESCA);
}

int pxerbla(int *icontxt, char *txt, int *info)
{
  return 0;
}

#endif

#if defined(LINUX) || defined(SUN)
/* this should be portable to other architectures.
 * note that on Crays the character length field comes directly 
 * after the character and not at the end of the function as for most other 
 * architectures.
 */

#if WRAP_BLACS
void cblacs_pinfo(int *mypnum, int *nprocs)
{
  blacs_pinfo__(mypnum, nprocs);
  return;
}

void cblacs_get(int icontxt, int what, int *val)
{
  blacs_get__(&icontxt, &what, val);
  return;
}

void cblacs_gridinit(int *icontxt, char *order, int nprow, int npcol)
{
  blacs_gridinit__(icontxt, order, &nprow, &npcol, 1L);
  return; 
}

void cblacs_gridinfo(int icontxt, int *nprow, int *npcol, 
		     int *myprow, int *mypcol)
{
  blacs_gridinfo__(&icontxt, nprow, npcol, myprow, mypcol); 
  return;
}

void cblacs_gridexit(int icontxt)
{
  blacs_gridexit__(&icontxt);
  return;
}

void cblacs_exit(int doneflag)
{
  blacs_exit__(&doneflag);
  return;
}
#endif

void cpdsygvx(int ITYPE, char *JOBZ, char *RANGE, char *UPLO, int N, 
	     double *A, int IA, int JA, int *DESCA, 
	     double *B, int IB, int JB, int *DESCB, 
	     double VL, double VU, int IL, int IU, 
	     double ABSTOL, int *M, int *NZ, double *W, 
	     double ORFAC, double *Z, int IZ, int JZ, int *DESCZ, 
	     double *WORK, int LWORK, int *IWORK, int LIWORK, 
	     int *IFAIL, int *ICLUSTR, double *GAP, int *INFO)
{

  pdsygvx_(& ITYPE, JOBZ, RANGE, UPLO, &N, A, &IA, &JA, DESCA,
	   B, &IB, &JB, DESCB, &VL, &VU, &IL, &IU, &ABSTOL, M, NZ,
	   W, &ORFAC, Z, &IZ, &JZ, DESCZ, WORK, &LWORK, IWORK, &LIWORK,
	   IFAIL, ICLUSTR, GAP, INFO, 1L, 1L, 1L);
  return;
}

int cnumroc(int N, int NB, int IPROC, int ISRCPROC, int NPROCS)
{
  return
    numroc_(&N, &NB, &IPROC, &ISRCPROC, &NPROCS);
}

int cpdelset(double *A, int IA, int JA, int *DESCA, double ALPHA)
{
  return 
    pdelset_(A, &IA, &JA, DESCA, &ALPHA);
}

int cpdelget(char *SCOPE, char *TOP, double *ALPHA, 
	     double *A, int IA, int JA, int *DESCA)
{
  return
    pdelget_(SCOPE, TOP, ALPHA, A, &IA, &JA, DESCA, 1L, 1L);
}

void cpdgemv(char *TRANSA, int M, int N, double ALPHA, 
	    double *A, int IA, int JA, int *DESCA, 
	    double *X, int IX, int JX, int *DESCX, int INCX, double BETA, 
	    double *Y, int IY ,int JY, int *DESCY, int INCY)
{
  pdgemv_(TRANSA, &M, &N, &ALPHA, A, &IA, &JA, DESCA, 
	  X, &IX, &JX, DESCX, &INCX, &BETA,
	  Y, &IY, &JY, DESCY, &INCY, 1L);
  return;
}

void cpdpotrf(char *UPLO, int N, double *A, int IA, int JA, 
	     int *DESCA, int *INFO)
{
  pdpotrf_(UPLO, &N, A, &IA, &JA, DESCA, INFO, 1L);
  return;
}

void cpdpotrs(char *UPLO, int N, int NRHS, double *A, int IA, int JA,
	      int *DESCA, double *B, int IB, int JB, int *DESCB, int *INFO)
{
  pdpotrs_(UPLO, &N, &NRHS, A, &IA, &JA, DESCA, B, &IB, &JB, DESCB, 
	   INFO, 1L);
  return;
}

void cpdtrtrs(char *UPLO, char *TRANS, char *DIAG, int N, int NRHS, 
	      double *A, int IA, int JA, int *DESCA,
	      double *B, int IB, int JB, int *DESCB, int *INFO)
{
  pdtrtrs_(UPLO, TRANS, DIAG, &N, &NRHS, A, &IA, &JA, DESCA,
	   B, &IB, &JB, DESCB, INFO, 1L, 1L, 1L);
  return;
}

#endif
