/* 
 *  C wrapper routines for ScaLapack 
 *  Time-stamp: <23 Oct 2002 17:44 BST cscalapack.h maising> 
 *  HISTORY
 *     23-11-2001   KSM   original version
 */ 
/* 
 * IMPORTANT: this file (and cscalapack.c) needs to be modified to suit 
 *   your computer architecture!
 *   It has only been tested on an Athlon cluster running Linux. 
 *   I'm not even sure if the Linux version is entirely correct.
 *   Most of the problems stem from the braindead decision of the 
 *   LAPACK / BLACS / SCALAPACK authors to use character arguments, 
 *   which are handled differently by different compilers, and
 *   mixed C/Fortran code within one library (BLACS, PBLAS: C,
 *   SCALAPACK: Fortran).
 */

#ifndef __CSCALAPACK_H
#define __CSCALAPACK_H

/* 
 * Scalapack blocksize. MADCAP passes this as a command line argument,
 * but I have no inclination to play around with it. Try other values
 * at your own leisure.
 */
#define BLOCKSIZE 64		

/*
 * here are comes a stab at porting to AIX. This has never been tested,
 * but was known to compile at some stage.
 */
#ifdef AIX

/* 
 * IBM's parallel scientific library "pessl" contains:
 *    pdpotrf, pdgemv, pdsygvx, numroc
 *  missing for no apparent reason: pdtrtrs, pdelset, pdelget, descinit 
 */
#include <pessl.h>

#define cnumroc numroc
#define cpdsygvx pdsygvx
#define cpdgemv pdgemv
#define cpdpotrf pdpotrf

extern int descinit(int *DESC, int *M, int *N, int *MB, int *NB, 
		    int *IRSRC, int *ICSRC, int *ICTXT, int *LLD, int *INFO);
extern int pdelget(char *SCOPE, char *TOP, double *ALPHA, 
		   void *A, int *IA, int *JA, int *DESCA);
extern int pdelset(void *A, int *IA, int *JA, int *DESCA, double *ALPHA);
extern int pxerbla(int *, char *, int *);
void cpdtrtrs(char *UPLO, char *TRANS, char *DIAG, int N, int NRHS, 
	      void *A, int IA, int JA, int *DESCA,
	      void *B, int IB, int JB, int *DESCB, int *INFO);

#endif /* AIX */


/*
 * Solaris and Linux version. Both seem to handle Fortran character
 * arguments in a similar way.
 */
#if defined(LINUX) || defined(SUN)

typedef long int ftnlen;	/* WARNING: check this on your machine! */
#if 0
typedef long int integer;
typedef double doublereal;
#else
/* I prefer defines because they circumvent the type-checking */
#define integer int		
#define doublereal double
#endif

#define WRAP_BLACS 1		/* use Fortran Blacs? */

#if ! WRAP_BLACS
#define cblacs_pinfo Cblacs_pinfo
#define cblacs_get Cblacs_get
#define cblacs_gridinit Cblacs_gridinit
#define cblacs_gridinfo Cblacs_gridinfo
#define cblacs_gridexit Cblacs_gridexit
#define cblacs_exit Cblacs_exit
#endif

#define descinit descinit_

/* Fortran headers */
#if WRAP_BLACS
extern int blacs_pinfo__(int *, int *);
extern int blacs_get__(int *, int *, int*);
extern int blacs_gridinit__(int *, char *, int *, int *, ftnlen); 
               /* FIXME: no ftnlen argument here? */
extern int blacs_gridinfo__(int *, int *, int *, int *, int *); 
extern int blacs_gridexit__(int *);
extern int blacs_exit__(int *);
#endif

/* this looks just like the next functions, but is written in C! */
extern void pdgemv_(char *TRANS, int * M, int * N, double * ALPHA,
		    double * A, int * IA, int * JA, int * DESCA,
		    double * X, int * IX, int * JX, int * DESCX, int * INCX,
		    double * BETA,
		    double * Y, int * IY, int * JY, int * DESCY, int * INCY,
		    ftnlen TRANS_LEN); /* FIXME: no ftnlen arg? */

/* now some Fortran functions */
extern int numroc_(int *, int *, int *, int *, int *);
extern int descinit_(int *, int *, int *, int *, int *, int *, int *, int *,
		    int *, int *);
extern int pdelset_(double *, int *, int *, int *, double *);
extern int pdelget_(char *, char *, double *, double *, int *, int *, int *,
		    ftnlen, ftnlen);
/* the next three headers came straight out of "f2c -A" */
extern int pdpotrf_(char *uplo, integer *n, 
		    doublereal *a, integer * ia, integer *ja, integer *desca, 
		    integer *info, ftnlen uplo_len);
extern int pdpotrs_(char *uplo, integer *n, integer *nrhs, 
		    doublereal *a, integer *ia, integer *ja, integer *desca, 
		    doublereal *b, integer *ib, integer *jb, integer *descb, 
		    integer *info, ftnlen uplo_len);
extern int pdtrtrs_(char *uplo, char *trans, char *diag, integer *n,
		    integer *nrhs, 
		    doublereal *a, integer *ia, integer *ja, integer *desca, 
		    doublereal *b, integer *ib, integer *jb, integer *descb, 
		    integer *info, 
		    ftnlen uplo_len, ftnlen trans_len, ftnlen diag_len);
extern int pdsygvx_(integer *ibtype, char *jobz, char *range, char *uplo, 
		    integer *n, 
		    doublereal *a, integer *ia, integer *ja, integer *desca, 
		    doublereal *b, integer *ib, integer *jb, integer *descb, 
		    doublereal *vl, doublereal *vu, integer *il, integer *iu, 
		    doublereal * abstol, integer *m, integer *nz, 
		    doublereal *w, doublereal *orfac, 
		    doublereal *z, integer *iz, integer *jz, integer *descz, 
		    doublereal * work, integer *lwork, integer *iwork, 
		    integer *liwork, integer *ifail, integer *iclustr, 
		    doublereal *gap, integer *info, ftnlen jobz_len, 
		    ftnlen range_len, ftnlen uplo_len);

/* C headers */
void cblacs_pinfo(int *mypnum, int *nprocs);
void cblacs_get(int icontxt, int what, int *val);
void cblacs_gridinit(int *icontxt, char *order, int nprow, int npcol) ;
void cblacs_gridinfo(int icontxt, int *nprow, int *npcol, 
		     int *myprow, int *mypcol);
void cblacs_gridexit(int icontxt);
void cblacs_exit(int doneflag);
int cnumroc(int N, int NB, int IPROC, int ISRCPROC, int NPROCS);
int cpdelset(double *A, int IA, int JA, int *DESCA, double ALPHA);
int cpdelget(char *SCOPE, char *TOP, double *ALPHA, 
	     double *A, int IA, int JA, int *DESCA);
void cpdgemv(char *TRANSA, int M, int N, double ALPHA, 
	     double *A, int IA, int JA, int *DESCA, 
	     double *X, int IX, int JX, int *DESCX, int INCX, double BETA, 
	     double *Y, int IY ,int JY, int *DESCY, int INCY);
void cpdpotrf(char *UPLO, int N, double *A, int IA, int JA, 
	      int *DESCA, int *INFO);
void cpdpotrs(char *UPLO, int N, int NRHS, double *A, int IA, int JA,
	      int *DESCA, double *B, int IB, int JB, int *DESCB, int *INFO);
void cpdtrtrs(char *UPLO, char *TRANS, char *DIAG, int N, int NRHS, 
	      double *A, int IA, int JA, int *DESCA,
	      double *B, int IB, int JB, int *DESCB, int *INFO);
void cpdsygvx(int ITYPE, char *JOBZ, char *RANGE, char *UPLO2, int N, 
	      double *A, int IA, int JA, int *DESCA, 
	      double *B, int IB, int JB, int *DESCB, 
	      double VL, double VU, int IL, int IU, 
	      double ABSTOL, int *M, int *NZ, double *W, 
	      double ORFAC, double *Z, int IZ, int JZ, int *DESCZ, 
	      double *WORK, int LWORK, int *IWORK, int LIWORK, 
	      int *IFAIL, int *ICLUSTR, double *GAP, int *INFO);

#endif
#endif

