c-----*-----------------------------------------------------------------

c program to interpolate uv data onto a square grid 
c KSM, 2001 (based on bin_vis.f)
c

c NOTES: to recreate the old binvis program:
c 1) set nbins even and greater than or equal to 400
c 2) set uvrange=uvcell*float(nbins)/2.
c 3) use decide subroutine rather than gridposn

      program  bin_wis
      implicit none
      integer  nvmax
      parameter (nvmax=500000)
      real*4   upt(nvmax),vpt(nvmax),wpt(nvmax)
      real*4   visr(nvmax),visi(nvmax),rms(nvmax)

      integer  nmax
      parameter (nmax=1001)   ! set nmax even to recreate old binvis
      integer  nwmax
      parameter (nwmax=5)
      integer  npts(nmax,nmax,nwmax),nwbins(nmax,nmax)
      real*4   upos(nmax,nmax,nwmax),vpos(nmax,nmax,nwmax)
      real*4   wpos(nmax,nmax,nwmax)
      real*4   sumr(nmax,nmax,nwmax),sumi(nmax,nmax,nwmax)
      real*4   sumrsq(nmax,nmax,nwmax),sumisq(nmax,nmax,nwmax)
      real*4   meanr(nmax,nmax,nwmax),meani(nmax,nmax,nwmax)
      real*4   weight(nmax,nmax,nwmax),sdev(nmax,nmax,nwmax)
      real*4   wmin(nmax,nmax),wmax(nmax,nmax)
      real*4   dummy(nmax,nmax)
      real*4   ubinplot(nmax*nmax),vbinplot(nmax*nmax)
      integer  hist(nwmax+2)

      integer  nbins
      integer  i,j,k,ii,count,nread,nfull,idum,nvmin,imirror,nold
      integer  ibin,inoise,ishift,iwrite,lnblnk,intype,outtype
      real*4   u,v,w,vr,vi,vrms,vabs,vabsmax,vrmsmax
      real*4   wdiff,wcell
      real*4   tempr,tempi,rmin,rmax
      real*4   pi,dtor,uvmin,uvmax,uvdist,uvrange,uvcell
      real*4   xmin,xmax,ymin,ymax,trans(6)
      character  inputfn*80,outputfn*80

c initialise some variables

      pi = 4.*atan(1.)            ! this equals pi
      dtor = pi / 180.            ! degrees/radians conversion

c ask for input uv file name

      write(*,'(a,$)') 'Input visibilities file name: '
      read(*,'(a)') inputfn
      open(unit=15, file=inputfn, status='OLD')     

c read in values of U, V, VISR, VISI and VRMS from input file
c assume units are lambda and Jy

      uvmin=1e10
      uvmax=0.
      vabsmax=0.
      vrmsmax=0.
      nread=0
 101  read(15,*,end=102) idum,u,v,w,vr,vi,vrms
      nread=nread+1
      if (nread.gt.nvmax) then
        write(*,*)
        write(*,*) ' ERROR: more than ',nvmax,' data points in file'
        write(*,*)
        stop
      else
        uvdist=sqrt(u**2+v**2)
        if (uvdist.lt.uvmin) uvmin=uvdist
        if (uvdist.gt.uvmax) uvmax=uvdist
        vabs=sqrt(vr**2+vi**2)
        if (vabs.gt.vabsmax) vabsmax=vabs
        if (vrms.gt.vrmsmax) vrmsmax=vrms
        upt(nread)=u
        vpt(nread)=v
        wpt(nread)=w
        visr(nread)=vr
        visi(nread)=vi
        rms(nread)=vrms   ! note now rms on real or imaginary part
      end if
      goto 101
 102  continue
      write(*,*)
      write(*,'(a,i6,a)') ' read ',nread,' data points'
      write(*,*)
      close(unit=15)
      
      write(*,*) 'uvmin, uvmax = ',uvmin,uvmax
      write(*,*) 'maximum visibility modulus   = ',vabsmax
      write(*,*) 'maximum visibility noise rms = ',vrmsmax
      write(*,*)

c start up pgplot and plot uv tracks

      call pgbegin(0,'?',1,1)
      call pgsci(1)
      uvrange=uvmax*1.3
      xmin=-uvrange
      xmax=uvrange
      ymin=-uvrange
      ymax=uvrange
      call pgenv(xmin,xmax,ymin,ymax,1,-2)
      call pgsci(2)
      call pgbox('ABCTSN',0.0,0,'ABCTSNV',0.0,0)
      call pgsci(3)
      call pglabel('\\fiu \\fr(wavelengths)',
     &             '\\fiv \\fr(wavelengths)',' ')
      call pgsci(6)
      call pgpoint(nread,upt,vpt,1)
      write(*,'(a,$)') 'Bin these data (1) or not (0): '
      read(*,*) ibin
 200  if (ibin.eq.0) then
        call pgend
        stop
      endif

c ask for uv cell size

      write(*,'(a,$)') 'Input uv cell size (wavelengths): '
      read(*,*) uvcell
      write(*,'(a,$)') 'Input w cell size (wavelengths): '
      read(*,*) wcell
      nbins=int(2.*1.1*uvmax/uvcell)
      if (mod(nbins,2) .eq. 1) then
         nbins=nbins+1
      endif
      if (nbins .gt. nmax) then
         write(*,*) 'FATAL ERROR: array size too small. ',
     &        'Program will crash!!!'
      endif

      uvrange=uvcell*float(nbins-1)/2.   ! if nbins is odd
c      uvrange=uvcell*float(nbins)/2.      ! if nbins is even
      write(*,*)
      write(*,*) '  uvmax, uvcell, nbins = ', uvmax, uvcell, nbins
      write(*,'(a,i4,a,i4,a,i4,a)') ' using ',nbins,' x ',nbins,
     &  ' x ', nwmax, ' bins '
      write(*,'(a,f6.2,a,f6.2,a,f6.2,a)') '  with cell size = ',
     &     uvcell,' x ',uvcell,' x ',wcell,' wavelengths'
      write(*,*)

c ask whether to use noise estimates or to calculate them from data

      write(*,'(a,$)') 
     &  'Use noise estimates in file (1) or calculate from data (0): '
      read(*,*) inoise
      if (inoise.eq.1) then
        nvmin=1
        write(*,'(a,$)') 
     &  'Are noise estimates in file on Re/Im part (1) or modulus (0): '
        read(*,*) intype
      else 
        write(*,'(a,$)') 'Input minimum no. of points per bin: '
        read(*,*) nvmin
      end if

      write(*,'(a,$)') 'Output noise on Re/Im part (1) or modulus (0): '
      read(*,*) outtype

c ask whether to use shift bin centres

      write(*,'(a,$)') 'Shift nominal bin positions (1) or not (0): '
      read(*,*) ishift

c ask whether to mirror uv-plane

      write(*,'(a,$)') 'Mirror uv-plane (1) or not (0): '
      read(*,*) imirror



c ask whether to write out binned values

      write(*,'(a,$)') 'Write out binned values (1) or not (0): '
      read(*,*) iwrite

c initialise arrays to zero

      do i=1,nbins
        do j=1,nbins
           do k=1,nwmax
              npts(i,j,k)=0
              upos(i,j,k)=0.
              vpos(i,j,k)=0.
              wpos(i,j,k)=0.
              sumr(i,j,k)=0.
              sumi(i,j,k)=0.
              weight(i,j,k)=0.
              sumrsq(i,j,k)=0.
              sumisq(i,j,k)=0.
           end do
           dummy(i,j)=0.
           wmin(i,j)=0.
           wmax(i,j)=0.
        end do
      end do

c calculate array elements

      do k=1,nwmax+2
         hist(k) = 0
      end do
      do ii=1,nread
         u = upt(ii)
         v = vpt(ii)
         w = wpt(ii)
         vr=visr(ii)
         vi=visi(ii)
         vrms=rms(ii)
c reduce uv-plane to one half
         if (imirror.eq.1) then
            if (u.lt.0.) then
               u=-u
               v=-v
               w=-w
               vi=-vi
            end if
         end if
c scale noises to Jy per sin and cos, not per modulus of visibility
         if (intype.eq.0) vrms=vrms/sqrt(2.)
         call gridposn(i,j,u,v,nbins,nbins,uvcell)
         
         if (w.lt.wmin(i,j)) wmin(i,j)=w
         if (w.gt.wmax(i,j)) wmax(i,j)=w
         npts(i,j,1) = npts(i,j,1)+1
      end do

c centre w-bins
      nold=0
      do i=1,nbins
         do j=1,nbins
            nwbins(i,j)=int((wmax(i,j)-wmin(i,j))/wcell)+1
            if (npts(i,j,1) .gt. 0) then
               hist(nwbins(i,j)+1) = hist(nwbins(i,j)+1)+1
               npts(i,j,1)=0
               nold=nold+1
            else
               hist(1) = hist(1)+1
            endif
            if (nwbins(i,j) .gt. nwmax) then
               write(*,*) 'More bins in w direction than allocated: ', 
     &              i, j, nwbins(i,j), nwmax  
            endif
            if (nwbins(i,j) .lt. 1) then
               write(*,*) 'Need at least 1 bin: ', 
     &              i, j, nwbins(i,j)
            endif
            
            wdiff=nwbins(i,j)*wcell - (wmax(i,j)-wmin(i,j))
            wmin(i,j)=wmin(i,j)-wdiff/2.
            wmax(i,j)=wmax(i,j)+wdiff/2.
         end do
      end do
      do k=1,nwmax+2
         write(*,*) 'pixels with ',k-1,' w-bins: ', hist(k)
      end do

      do ii=1,nread
        u=upt(ii)
        v=vpt(ii)
        w=wpt(ii)
        vr=visr(ii)
        vi=visi(ii)
        vrms=rms(ii)
        if (imirror.eq.1) then
           if (u.lt.0.) then
              u=-u
              v=-v
              w=-w
              vi=-vi
           end if
        end if
c ... decide in which bin this uv point belongs
        call gridposn(i,j,u,v,nbins,nbins,uvcell)
c ... determine w bin
        k=int((w-wmin(i,j))/(wcell))+1

c        call decide(i,u,uvrange,nbins)   ! use decide to recreate old binvis
c        call decide(j,v,uvrange,nbins)   ! use decide to recreate old binvis
c ... update arrays
        npts(i,j,k)=npts(i,j,k)+1
c use noise estimate from file
        if (inoise.eq.1) then
          if (vrms.le.0.) then
            upos(i,j,k)=upos(i,j,k)+u
            vpos(i,j,k)=vpos(i,j,k)+v
            wpos(i,j,k)=wpos(i,j,k)+w
            sumr(i,j,k)=sumr(i,j,k)+vr
            sumi(i,j,k)=sumi(i,j,k)+vi
          else
c calculate weighted means
            upos(i,j,k)=upos(i,j,k)+u/(vrms**2)
            vpos(i,j,k)=vpos(i,j,k)+v/(vrms**2)
            wpos(i,j,k)=wpos(i,j,k)+w/(vrms**2)
            sumr(i,j,k)=sumr(i,j,k)+vr/(vrms**2)
            sumi(i,j,k)=sumi(i,j,k)+vi/(vrms**2)
            weight(i,j,k)=weight(i,j,k)+(1.0/(vrms**2))
          end if
c calculate noise estimate from data
        else
          upos(i,j,k)=upos(i,j,k)+u
          vpos(i,j,k)=vpos(i,j,k)+v
          wpos(i,j,k)=wpos(i,j,k)+w
          sumr(i,j,k)=sumr(i,j,k)+vr
          sumi(i,j,k)=sumi(i,j,k)+vi
          sumrsq(i,j,k)=sumrsq(i,j,k)+vr**2
          sumisq(i,j,k)=sumisq(i,j,k)+vi**2
        end if
      end do

c calculate mean and rms of bins

      nfull=0
      do i=1,nbins
         do j=1,nbins
            do k=1,nwbins(i,j)
               if (npts(i,j,k).le.0) then
                  meanr(i,j,k)=0.
                  meani(i,j,k)=0.
                  sdev(i,j,k)=0.
               else
                  if (inoise.eq.1) then
c use noise from file
                     nfull=nfull+1
                     if (weight(i,j,k).gt.0.) then
                        upos(i,j,k)=upos(i,j,k)/weight(i,j,k)
                        vpos(i,j,k)=vpos(i,j,k)/weight(i,j,k)
                        wpos(i,j,k)=wpos(i,j,k)/weight(i,j,k)
                        meanr(i,j,k)=sumr(i,j,k)/weight(i,j,k)
                        meani(i,j,k)=sumi(i,j,k)/weight(i,j,k)
                        sdev(i,j,k)=sqrt(1.0/weight(i,j,k))
                     else
                        upos(i,j,k)=upos(i,j,k)/real(npts(i,j,k))
                        vpos(i,j,k)=vpos(i,j,k)/real(npts(i,j,k))
                        wpos(i,j,k)=wpos(i,j,k)/real(npts(i,j,k))
                        meanr(i,j,k)=sumr(i,j,k)/real(npts(i,j,k))
                        meani(i,j,k)=sumi(i,j,k)/real(npts(i,j,k))
                        sdev(i,j,k)=0.
                     end if
                  else
c estimate noise from data
                     if (npts(i,j,k).ge.nvmin) then
                        nfull=nfull+1
                        upos(i,j,k)=upos(i,j,k)/real(npts(i,j,k))
                        vpos(i,j,k)=vpos(i,j,k)/real(npts(i,j,k))
                        wpos(i,j,k)=wpos(i,j,k)/real(npts(i,j,k))
                        meanr(i,j,k)=sumr(i,j,k)/real(npts(i,j,k))
                        meani(i,j,k)=sumi(i,j,k)/real(npts(i,j,k))
                        tempr=sumrsq(i,j,k)/real(npts(i,j,k))
     &                       -meanr(i,j,k)**2
                        tempr=tempr/real(npts(i,j,k)-1)
                        tempr=sqrt(tempr)
                        tempi=sumisq(i,j,k)/real(npts(i,j,k))
     &                       -meani(i,j,k)**2
                        tempi=tempi/real(npts(i,j,k)-1)
                        tempi=sqrt(tempi)
                        sdev(i,j,k)=(tempr+tempi)/2.
                     else
                        meanr(i,j,k)=0.
                        meani(i,j,k)=0.
                        sdev(i,j,k)=0.
                     end if
                  end if
               end if
c use grid centres instead of centre of gravity
               if (ishift.eq.0) then
                  call coords(i,j,u,v,nbins,nbins,uvcell)
                  upos(i,j,k)=u
                  vpos(i,j,k)=v
                  wpos(i,j,k)=wmin(i,j)+(real(k)-0.5)*wcell
               end if
            end do
         end do
      end do

c adjust noises as required

      if (outtype.eq.0) then
        do i=1,nbins
          do j=1,nbins
             do k=1,nwbins(i,j)
                sdev(i,j,k)=sdev(i,j,k)*sqrt(2.)
             end do
          end do
        end do
      end if

c write out information about bins and set array for plotting

      write(*,*)
      if (iwrite.eq.1) then
        write(*,*)
        write(*,*) ' non-zero bins: i, j, npts, u,v, visr, visi, rms'
        write(*,*)
      end if
      count=0
      do i=1,nbins
         do j=1,nbins
            do k=1,nwbins(i,j)
               if (npts(i,j,k).ge.nvmin) then
                  if (iwrite.eq.1) then
                     write(*,*) i,j,npts(i,j,k),upos(i,j,k),vpos(i,j,k),
     &                    meanr(i,j,k),meani(i,j,k),sdev(i,j,k)
                  end if
                  count=count+1
c this may plot multiple points at the same place
                  ubinplot(count)=upos(i,j,k)
                  vbinplot(count)=vpos(i,j,k)
               end if
            end do
         end do
      end do

      if (count.ne.nfull) pause 'bin number error'
      write(*,*)
      write(*,*) ' total number of bins    = ',nbins*nbins
      write(*,*) ' number of non-zero bins = ',nfull,' (',nold,')'

c find min and max elements of MEANARR array and fill DUMMY array
c this can't do more than one w bin

      rmax=0.
      rmin=1.0e30
      do i=1,nbins 
        do j=1,nbins
          if (npts(i,j,1).gt.0) then
            dummy(i,j)=sqrt(meanr(i,j,1)**2+meani(i,j,1)**2)
            if (dummy(i,j).gt.rmax) rmax=dummy(i,j)
            if (dummy(i,j).lt.rmin) rmin=dummy(i,j)
          end if
        end do
      end do
      write(*,*) ' max and min visibility modulus = ',rmax,rmin
      write(*,*)

c define transformation from array element i,j to world co-ordinates

      trans(1)=-uvrange-uvcell
      trans(2)=uvcell
      trans(3)=0
      trans(4)=-uvrange-uvcell
      trans(5)=0
      trans(6)=uvcell

      xmin=-uvmax*1.3
      xmax=uvmax*1.3
      ymin=-uvmax*1.3
      ymax=uvmax*1.3
      call pgenv(xmin,xmax,ymin,ymax,1,-2)
      call pggray(dummy,nbins,nbins,1,nbins,1,nbins,rmin,rmax,trans)
      call pgsci(2)
      call pgbox('ABCTSN',0.0,0,'ABCTSNV',0.0,0)
      call pgsci(3)
      call pglabel('u','v','Absolute value of visibilities')
      call pgsci(4)
      call pgpoint(count,ubinplot,vbinplot,1)

c ask for output file name

      write(*,'(a,$)') 'Output file name (.wbn added) '
      read(*,'(a)') inputfn

c open uv positions, uv data and uv noise files

      outputfn=inputfn(1:lnblnk(inputfn)) // '.wbn'
      open(unit=16, file=outputfn,status='UNKNOWN')

c write arrays to files and create noise array in lambda and Jy 

      idum=0
      do i=1,nbins
         do j=1,nbins
            do k=1,nwbins(i,j)
               if (npts(i,j,k).ge.nvmin) then
                  idum=idum+1
                  write(16,'(i6,6f14.6)') idum,upos(i,j,k),
     &                 vpos(i,j,k),wpos(i,j,k),
     &                 meanr(i,j,k),meani(i,j,k),sdev(i,j,k)
               end if
            end do
         end do
      end do
      close(unit=16)

c ask whether to re-bin

      write(*,*)
      write(*,'(a,$)') 'Re-bin these data (1) or not (0): '
      read(*,*) ibin
      goto 200

      end

c-----*-----------------------------------------------------------------

      subroutine decide(i,u,uv_range,nbins)
      implicit real (a-h,o-z)

      rinterval=2.*uv_range/nbins
      i=(u+uv_range)/rinterval+1

      end

c-----*-----------------------------------------------------------------

c subroutine to find nearest grid position (i,j) from co-ords (x,y)

      subroutine gridposn(i,j,x,y,nx,ny,cell)
      implicit  none
      integer*4 i,j,nx,ny
      real*4    x,y,cell

      integer*4 icentre,jcentre,ifail
      real*4    signx,signy

      ifail=0

      signx=1.
      if (x.lt.0.) signx=-1.
      signy=1.
      if (y.lt.0.) signy=-1.

      icentre=nx/2+1
      jcentre=ny/2+1
      i=int(x/cell+signx*0.5)+icentre
      j=int(y/cell+signy*0.5)+jcentre

      if ((i.lt.1).or.(i.gt.nx)) ifail=1
      if ((j.lt.1).or.(j.gt.ny)) ifail=1
      if (ifail.eq.1) pause ' ERROR - IFAIL = 1 in GRIDPOSN'

      return
      end

c-----*-----------------------------------------------------------------

c subroutine to find co-ords (x,y) from grid position (i,j)

      subroutine coords(i,j,x,y,nx,ny,cell)
      implicit  none
      integer*4 i,j,nx,ny
      real*4    x,y,cell

      integer*4 icentre,jcentre

      icentre=nx/2+1
      jcentre=ny/2+1
      x=float(i-icentre)*cell
      y=float(j-jcentre)*cell

      return
      end

c-----*-----------------------------------------------------------------



