c-----*----------------------------------------------------------------

      program vis2ps
      implicit none
      integer nbinmax
      parameter (nbinmax=1500)
      integer num(nbinmax)
      real    ps(nbinmax),pserr(nbinmax)

      integer nvismax
      parameter (nvismax=100000)
      real    upos(nvismax),vpos(nvismax)
      complex vis(nvismax)

      integer i,idum,iplot,ctype,nskip,nvis,nbins
      real    uobs,vobs,visr,visi,rms,beamsqarea,fwhm,sigma
      real    pi,degtorad,freq,tcmb,jy_to_uk
      real    st,bw
      character infile*80

      real     dt_to_di
      external dt_to_di

c initialise variables

      pi=3.14159265
      degtorad=pi/180.
      tcmb=2.726

c open visibilities file

      write(*,'(a,$)') ' Input visibilities file name: '
      read(*,'(a)') infile
      open(15, file=infile, status='OLD', form='FORMATTED')

      write(*,'(a,$)') ' Input data points skip (0 = no skip): '
      read(*,*) nskip

      write(*,'(a,$)') ' Input observing frequency (GHz): '
      read(*,*) freq
      jy_to_uk=1.0/(1.0e-6*dt_to_di(tcmb,freq)*1.0e26)

c copy into array

      nvis=0
 100  continue
      do i=1,nskip
        read(15,*,end=500) idum,uobs,vobs,visr,visi,rms
      end do
      read(15,*,end=500) idum,uobs,vobs,visr,visi,rms
      nvis=nvis+1
      upos(nvis)=uobs      
      vpos(nvis)=vobs      
      vis(nvis)=cmplx(visr,visi)*jy_to_uk
      goto 100
 500  continue

c calculate PS 

      write(*,*) 'Calculating power spectrum ... '

      write(*,'(a,$)') ' Input centre of first bin in multipoles: '
      read(*,*) st
      write(*,'(a,$)') ' Input width of bin in multipoles: '
      read(*,*) bw
      write(*,'(a,$)') ' Input number of PS bins to plot: '
      read(*,*) nbins

      st=st/2.0/pi
      bw=bw/2.0/pi

c ... calculate area of (primary-beam)^2
      write(*,'(a,$)') ' Input FWHM of primary beam (degrees): '
      read(*,*) fwhm
      sigma=fwhm/(2.*sqrt(2.*log(2.)))
      sigma=sigma*degtorad
      beamsqarea=pi*sigma**2

c ... calculate 1D PS
      call calc_ps(nvis,upos,vpos,vis,nbins,ps,pserr,num,st,bw,
     &             beamsqarea)

c ... plot calculated 1D PS
      write(*,*) 'Plotting power spectrum ... '
      call pgbegin(0,'?',1,1)
      call plot_ps(ps,pserr,st,bw,nbins)

c ... plot theoretical PS
      write(*,120)
 120  format(' Plot theoretical power spectrum (1) or not (0): ',$)
      read(*,*) iplot
      if (iplot.eq.1) then
        write(*,'(a,$)')' CDM (0), GACF (1), flat (2), power-law (3): ' 
        read(*,*) ctype
        call pgsls(2)
        call plot_theo(ctype)
      end if
      call pgend

 999  end

c-----*-----------------------------------------------------------------

      subroutine plot_theo(ctype)
      implicit none
      integer  ctype

      integer  nl
      parameter (nl=1500)
      integer  l,m
      real     pi,cl(nl),xarr(nl),yarr(nl)

      pi=3.14159265

      if (ctype.eq.0) then
        call read_dls
        call read_dlst
      end if

      call get_theops(nl,cl,ctype)
      m=0
      do l=2,nl
        m=m+1
c        xarr(m)=alog10(float(l))
        xarr(m)=(float(l))
        yarr(m)=float(l*(l+1))*cl(l)/(2.0*pi)
      end do
      call pgline(nl-1,xarr,yarr)

      return
      end

c-----*-----------------------------------------------------------------
 
      subroutine calc_ps(nvis,upos,vpos,vis,nbins,ps,pserr,num,
     &                   start,bw,beamsqarea)
      implicit none
      integer  nvis
      real     upos(nvis),vpos(nvis)
      complex  vis(nvis)
      integer  nbins
      integer  num(nbins)
      real     ps(nbins),pserr(nbins)
      real     start,bw,beamsqarea

      integer  i,bin
      real     pi,power,uvdist,rn,tcmb,factor

c ... initialise variables
      pi=3.14159265
      tcmb=2.726
      factor=1e-6/tcmb
      do i=1,nbins
        num(i)=0
        ps(i)=0.
        pserr(i)=0.
      end do

c ... calculate 1D power spectrum
      do i=1,nvis
         uvdist=sqrt(upos(i)**2+vpos(i)**2)
         call bin_number(bin,bw,start,uvdist)
         if (bin.gt.0) then
           num(bin)=num(bin)+1
           power=cabs(vis(i))**2
           power=power/beamsqarea
           power=power*(2.0*pi*uvdist)**2
           power=power/2.0/pi
           ps(bin)=ps(bin)+power
           pserr(bin)=pserr(bin)+power**2
         end if
      end do

      do bin=1,nbins
        if (num(bin).gt.1) then
          rn=float(num(bin))
          ps(bin)=ps(bin)/rn
          pserr(bin)=sqrt((pserr(bin)/rn-ps(bin)**2)/(rn-1.0))
          ps(bin)=ps(bin)*factor**2
          pserr(bin)=pserr(bin)*factor**2
          write(*,*) 'bin,ps,num = ',bin,ps(bin),num(bin)
        end if
      end do

      return
      end

c-----*-----------------------------------------------------------------

      subroutine plot_ps(ps,pserr,start,binwidth,nbins)
      implicit none
      integer  nbins
      real     ps(nbins),pserr(nbins),start,binwidth

      integer  i,lmin,lmax
      real     xmin,xmax,ymin,ymax
      real     pi,xpos,ypos,xlo,xhi,ylo,yhi,ell

      pi=3.14159265

c ... define limits of plot
c      write(*,110)
c 110  format(' Input multipole limits lmin, lmax: ',$)
c      read(*,*) lmin,lmax
c      xmin=alog10(real(lmin))
c      xmax=alog10(real(lmax))

      xmin=(start-0.5*binwidth)*2.0*pi
      xmax=(start+(nbins-0.5)*binwidth)*2.0*pi
      write(*,*) 'xmin: ', xmin, 'xmax: ', xmax

      write(*,120)
 120  format(' Input power spectrum limits (uK^2) (-1 auto): ',$)
      read(*,*) ymin,ymax
      if (ymin.lt.0.) ymin=0.
      if (ymax.lt.0.) ymax=1e-9

c ... set PGPLOT parameters
      call pgscf(2)
      call pgsch(1.1)
      call pgsls(1)
      call pgslw(1)
      call pgenv(xmin,xmax,ymin,ymax,0,0)
      call pglabel('\\fil', '\\fil(l+1)C\\dl\\u/2\\gp\\fr',' ')

c ... plot positions of measured points in PS realisation
      do i=1,nbins
        ell=(start+float(i-1)*binwidth)*2.0*pi
        if (ell.gt.0.) then
c          xpos=alog10(ell)
c          xlo=alog10(ell-binwidth*pi)
c          xhi=alog10(ell+binwidth*pi)
          xpos=(ell)
          xlo=(ell-binwidth*pi)
          xhi=(ell+binwidth*pi)
          ypos=ps(i)
          ylo=ps(i)-pserr(i)
          yhi=ps(i)+pserr(i)
          call pgpt1(xpos,ypos,17)
          call pgerrx(1,xlo,xhi,ypos,0.)
          call pgerry(1,xpos,ylo,yhi,0.)
        end if
      end do

      return
      end

c-----*-----------------------------------------------------------------

c subroutine to find radial bin number from radial coordinate

      subroutine bin_number(bin,binwidth,start,rad)
      implicit  none
      integer*4 bin
      real*4    binwidth,start,rad

      bin=int(abs(rad-start)/binwidth+0.5)+1

      return
      end

c-----*-----------------------------------------------------------------

      subroutine get_theops(nl,cl,ctype)
      implicit none
      integer  nl,ctype
      real     cl(nl)

      integer l,lmin,lmax
      real    pi,tcmb,degtorad,mintorad,dttm10,sc,qcobe
      real    h0,omegab,ns,tau,omegat,lambda,q,r,nt,omegabh2,dtt
      real    c0,thetac,arg,dlband
      real    dl,dln

      external dl,dln

c initialise some variables

      pi=3.14159265
      tcmb=2.726
      degtorad=pi/180.
      mintorad=degtorad/60.
      dttm10=1.833e-10
      sc=10.0
      qcobe=18.0

c use CDM power spectrum

      if (ctype.eq.0) then
        write(*,'(a,$)')' Input H0,Omegab,ns,tau,Omegat,Lambda,Q,r,nt: '
        read(*,*) h0,omegab,ns,tau,omegat,lambda,q,r,nt
        omegabh2=omegab*(h0/100.)**2
        dtt=dttm10*(q/qcobe)**2
        do l=2,nl
          arg=float(l)
  	  cl(l)=dl(l,h0,omegabh2,ns,tau,omegat,lambda,dtt,sc,r,nt)
          cl(l)=cl(l)/float(l*(l+1))
c          cl(l)=cl(l)*(tcmb*1.0e6)**2   ! put units into (uK)^2
        end do

c use GACF power spectrum

      else if (ctype.eq.1) then
        write(*,'(a,$)')' Input sqrt(C_0) and theta_c (uK,arcmin): '
        read(*,*) c0,thetac
        c0=c0**2
        thetac=thetac*mintorad
        do l=2,nl
          arg=float(l)
          cl(l)=2*pi*(thetac**2)*c0*exp(-(thetac**2)*arg*arg/2.)
          cl(l)=cl(l)/(tcmb*1.0e6)**2 
c          cl(l)=cl(l)*(tcmb*1.0e6)**2   ! put units into (uK)^2
        end do

c use flat dl band power spectrum

      else if (ctype.eq.2) then
        write(*,'(a,$)') ' Input flat dl band power: '
        read(*,*) dlband
        write(*,'(a,$)') ' Input limits of band in multipoles: '
        read(*,*) lmin,lmax
        do l=2,nl
          arg=float(l)
          if ((l.ge.lmin).and.(l.le.lmax)) then
            cl(l)=dlband
          else
            cl(l)=0.
          end if
          cl(l)=cl(l)/float(l*(l+1))
c          cl(l)=cl(l)*(tcmb*1.0e6)**2   ! put units into (uK)^2
        end do

c use power-law spectrum

      else if (ctype.eq.3) then
        write(*,'(a,$)') ' Input sqrt(C_0) at freq_0 (uK): '
        read(*,*) c0
        c0=c0**2
        do l=2,nl
          arg=float(l)
          cl(l)=4.0*pi*c0*(98./19.)*(1./(5.+arg))**3
          cl(l)=cl(l)/(tcmb*1.0e6)**2 
c          cl(l)=cl(l)*(tcmb*1.0e6)**2   ! put units into (uK)^2
        end do
      end if

      return
      end

c-----*-----------------------------------------------------------------

      subroutine gridposn(i,j,x,y,nx,ny,cell)
      implicit  none
      integer*4 i,j,nx,ny
      real*4    x,y,cell

      integer*4 icentre,jcentre
      real*4    signx,signy

      signx=1.
      if (x.lt.0.) signx=-1.
      signy=1.
      if (y.lt.0.) signy=-1.

      icentre=nx/2+1
      jcentre=ny/2+1
      i=int(x/cell+signx*0.5)+icentre
      j=int(y/cell+signy*0.5)+jcentre

      return
      end

c-----*-----------------------------------------------------------------

c calculates conversion factor between equivalent CMBR thermodynamic
c temperature fluctuation in K to intensity fluctuations in W/m^2/Hz/sr
c assuming frequency in GHz

      real function dt_to_di(tcmb,freq)
      implicit none
      real tcmb,freq

      real norm
      parameter (norm=6.675e-20)      ! k**3/(h**2 c**2) in SI units
      real cmbfcn

      external cmbfcn

      dt_to_di=2.0*(tcmb**2)*norm*cmbfcn(tcmb,freq)

      return
      end

c-----*-----------------------------------------------------------------

c intensity frequency dependence due to CMBR

      real function cmbfcn(tcmb,freq)
      implicit none
      real tcmb,freq

      real hk,x
      parameter (hk=0.0479927)        ! this is h/k for frequency in GHz

      x=hk*freq/tcmb
      cmbfcn=(x**4)*exp(x)/((exp(x)-1)**2)

      return
      end

c-----*-----------------------------------------------------------------

      include "dlssubn.f"
      include "dltsubn.f"
      include "dlfuncn.f"
      include "splines.f"

c-----*-----------------------------------------------------------------
