/*  Time-stamp: <12 Nov 2002 18:08 GMT like_integrals.h maising> */

#ifndef __LIKE_INTEGRALS_H
#define __LIKE_INTEGRALS_H

#include "integration.h"


#define N_BINS 41

#define INT_STORE_MEMORY 1
#define INT_STORE_DISK 2
#define INT_LOAD_ONLY 4
#define INT_SAVE_ONLY 8

#define INT_STATE_NULL 0
#define INT_STATE_MEMORY 1
#define INT_STATE_DISK 2

typedef struct {
  int nv;			/* number of visibilities */
  int nz;             /* number of non-zero matrix elements */
  int *irn, *icn;     /* index for non-zero elements of covariance matrix */
  int *freqn;
  int imaginary_flag;
  float *integral1r[N_BINS+1], *integral2r[N_BINS+1];
  float *integral1i[N_BINS+1], *integral2i[N_BINS+1];
  int n_bins;
  int *maxbin1, *maxbin2;
  int j1, j2;
} integral_struct;


void set_beamfft(dcomplex (*beamfft)(double, double, double));
int countintegrals(double *antwdth, int nf, 
		   int *freqn, double *u, double *v, int nv);
void newintegrals(integral_struct *data, int nv, int nz, int n_bins);
void freeintegrals(integral_struct *data);
void makeintegrals(integral_struct *common,
		   int *freqn, double *u, double *v, double *w, int nv, 
		   double *ra, double *dec, double ra_ref, double dec_ref,  
		   double *freq, double *antwdth, double *sigma, int nf,
		   double *rbins, int na, int gaussian_beam,
		   int imagflag, int rank, int npe);
void shrink_integrals(integral_struct *ints, int rank, int npe);
void saveintegrals(char filename[], integral_struct *ints, int rank, int npe);
void loadintegrals(char filename[], integral_struct *ints,
		   int *freqn, int rank, int npe);
void copyintegrals(integral_struct *int1, integral_struct *int2);
void get_mpi_integral_cols(int *icn, int nv, int nz, int npe, int *jidx);
void get_integrals(integral_struct *integral_data, 
		   int *freqn, double *u, double *v, double *w, int nv,
		   double *ra, double *dec, double ra_ref, double dec_ref, 
		   double *freq, double *antwdth, double *sigma, int nf,
		   double *rbins, int na, int bin, int field, int nfield,
		   int gaussian_beam,  int imagflag, int saveflag, 
		   char prefix[], int rank, int npe);
void reset_integrals(int na, int ni);
void dealloc_integrals(int na, int ni);

#if defined(FITSIO)
void savefitsintegrals(char filename[], integral_struct *ints);
void loadfitsintegrals(char filename[], integral_struct *ints, int *freqn,
		       double *freq, int nf);
#endif

#endif
