/*
 * this file contains routines for sorting visibilities by their 
 * polar coordinates in the uv-plane
 */
/*  Time-stamp: <11 Oct 2002 15:23 BST like_sort.c maising> */
/* HISTORY
 *   ??-05-01  KSM  original version
 */


#include <stdio.h>
#include <math.h>
#include "like_sort.h"
#include "my_malloc2.h"

#define PI 3.14159265358979323844
#define MALLOC1 malloc_1dmatrix
#define FREE1 free_1dmatrix
#define MALLOC1I malloc_1imatrix
#define FREE1I free_1imatrix

/*-----*-----------------------------------------------------------------*/

#define NRANSI
#define SWAP(a,b) itemp=(a);(a)=(b);(b)=itemp;
#define M 7
#define NSTACK 50

/*
 * indexes an array arr[], i.e., outputs the array indx[] such that
 * arr[indx[j]] is in ascending order.
 */
void mydindexx(unsigned long n, double arr[], unsigned long indx[])
{
	unsigned long i,indxt,ir=n,itemp,j,k,l=1;
	int jstack=0,*istack;
	double a;

	istack=MALLOC1I(NSTACK);
	for (j=1;j<=n;j++) indx[j]=j;
	for (;;) {
		if (ir-l < M) {
			for (j=l+1;j<=ir;j++) {
				indxt=indx[j];
				a=arr[indxt];
				for (i=j-1;i>=1;i--) {
					if (arr[indx[i]] <= a) break;
					indx[i+1]=indx[i];
				}
				indx[i+1]=indxt;
			}
			if (jstack == 0) break;
			ir=istack[jstack--];
			l=istack[jstack--];
		} else {
			k=(l+ir) >> 1;
			SWAP(indx[k],indx[l+1]);
			if (arr[indx[l+1]] > arr[indx[ir]]) {
				SWAP(indx[l+1],indx[ir])
			}
			if (arr[indx[l]] > arr[indx[ir]]) {
				SWAP(indx[l],indx[ir])
			}
			if (arr[indx[l+1]] > arr[indx[l]]) {
				SWAP(indx[l+1],indx[l])
			}
			i=l+1;
			j=ir;
			indxt=indx[l];
			a=arr[indxt];
			for (;;) {
				do i++; while (arr[indx[i]] < a);
				do j--; while (arr[indx[j]] > a);
				if (j < i) break;
				SWAP(indx[i],indx[j])
			}
			indx[l]=indx[j];
			indx[j]=indxt;
			jstack += 2;
			if (jstack > NSTACK) myerror("NSTACK too small in indexx.");
			if (ir-i+1 >= j-l) {
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} else {
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
	FREE1I(istack);
}
#undef M
#undef NSTACK
#undef SWAP
#undef NRANSI
/* (C) Copr. 1986-92 Numerical Recipes Software ?4210(9p#3. */

/*
 * sort visibilities by the value of the array angle
 */
void sort_vis(int *freqn, double *u, double *v, double *w,
	      double *visr, double *visi, double *rms, double *angle,
	      int nv)
{
  unsigned long *idx, i, n;
  int *fw;
  double *uw, *vw, *ww, *rw, *iw, *ew;

  if (nv <= 1)
    return;

  n = (unsigned long)nv;
  idx = (unsigned long *)malloc_1matrix(nv, sizeof(unsigned long));
  mydindexx(n, angle, idx);

  fw = MALLOC1I(nv);
  uw = MALLOC1(nv); vw = MALLOC1(nv); ww = MALLOC1(nv);
  rw = MALLOC1(nv); iw = MALLOC1(nv); ew = MALLOC1(nv);
  for (i = 1; i <= n; i++) fw[i] = freqn[i];
  for (i = 1; i <= n; i++) uw[i] = u[i]; 
  for (i = 1; i <= n; i++) vw[i] = v[i];
  for (i = 1; i <= n; i++) ww[i] = w[i];
  for (i = 1; i <= n; i++) rw[i] = visr[i]; 
  for (i = 1; i <= n; i++) iw[i] = visi[i]; 
  for (i = 1; i <= n; i++) ew[i] = rms[i]; 
    
  for (i = 1; i <= n; i++) freqn[i] = fw[idx[i]]; 
  for (i = 1; i <= n; i++) u[i] = uw[idx[i]]; 
  for (i = 1; i <= n; i++) v[i] = vw[idx[i]]; 
  for (i = 1; i <= n; i++) w[i] = ww[idx[i]]; 
  for (i = 1; i <= n; i++) visr[i] = rw[idx[i]]; 
  for (i = 1; i <= n; i++) visi[i] = iw[idx[i]]; 
  for (i = 1; i <= n; i++) rms[i] = ew[idx[i]]; 

  FREE1I(fw);
  FREE1(uw); FREE1(vw); FREE1(ww);
  FREE1(rw); FREE1(iw); FREE1(ew);
  free_1matrix((void *)idx, sizeof(unsigned long));

  return;
}


double angle(double u, double v, double w)
{
  double a;
  const double tiny = 1.e-10;
  
  if (u >= 0.) {
    if (v >= 0.)
      a = atan(v / (u + tiny));
    else
      a = atan(v / (u + tiny))+PI;
  }
  else {
    if (v >= 0.)
      a = atan(v / (u + tiny))+PI;
    else
      a = atan(v / (u + tiny));
  }
  return a;
}

double radius(double u, double v, double w)
{
  return sqrt(u*u + v*v);
}

/*
 * sort visibilities by their polar angle in the uv-plane
 */
void sort_data(int *freqn, double *u, double *v, double *w, 
	       double *visr, double *visi, double *rms, int nv,
	       double func(double, double, double))
{
  int i;
  double *major;
  
  if (nv <= 1)
    return;

  major = MALLOC1(nv);
  for (i = 1; i <= nv; i++) 
    major[i] = (*func)(u[i], v[i], w[i]);

  sort_vis(freqn, u, v, w, visr, visi, rms, major, nv);
  FREE1(major);

  return;
}
/*-----*-----------------------------------------------------------------*/
