/* 
 * Temperature and brightness dependency and conversion routines
 */
/*  Time-stamp: <8 Oct 2002 20:35 BST like_temp.c maising> */

#include <math.h>
#include "like_temp.h"

#define PI 3.14159265358979323844

/*-----*-----------------------------------------------------------------*/

/* 
 * intensity frequency dependence due to CMBR
 * PARAMS
 *     tcmb  [I]  temperature of the CMBR in K
 *     freq  [I]  frequency in GHz
 * RETURNS 
 */
double cmbfcn(double tcmb, double freq)
{
  double hk = 0.0479927;	/* this is h/k for frequency in GHz */
  double x;

  x = hk * freq / tcmb;
  return (pow(x, 4.) * exp(x) / ((exp(x)-1) * (exp(x)-1)));
}

/* 
 * calculates conversion factor between equivalent CMBR thermodynamic
 * temperature fluctuation in K to intensity fluctuations in W/m^2/Hz/sr
 * PARAMS
 *     tcmb  [I]  temperature of the CMBR in K
 *     freq  [I]  frequency in GHz
 * RETURNS 
 *     conversion factor from temperature in K to brightness in W/m^2/Hz/sr
 */
double dt_to_di(double t_cmb, double freq)
{
  double k3overh2c2 = 6.675e-20; /* k**3/(h**2 c**2) in SI units */
  double hoverk = 0.0479927;     /* h/k for frequency in GHz */
  double x;

  x = hoverk * freq / t_cmb;
  return 2. * (t_cmb*t_cmb) * k3overh2c2 * 
    pow(x,4.)*exp(x)/((exp(x)-1.)*(exp(x)-1.));
}

/*
 *
 * 1d PS are in l^2 C_l/2pi in dT/T units; since l=2pi u this means they
 * are in 2pi u^2 S[u] in dT/T units. Calculate conversion factor to
 * give u^2 S(u) in (Jy/sr)^2 at freq0.
 */
double ps_conversion(double tcmb, double freq0) 
{
  double convf;

  convf = tcmb * tcmb;   /*...convert to (K)^2 units*/
  convf *= (dt_to_di(tcmb, freq0)*1.e26) * (dt_to_di(tcmb, freq0)*1.e26);
  /*...convert to (Jy/sr)^2 units*/
  convf /= (2. * PI);  /*...divide by 2pi*/

  return convf;
}
