/*
 * matrix inversion routine from NR
 */
/*  Time-stamp: <19 Oct 2002 18:14 BST matinvert.c maising> */

#include <math.h>
#include "matinvert.h"
#include "my_malloc2.h"

void matinv_ludcmp(double **a, int n, int *indx, double *d);
void matinv_lubksb(double **a, int n, int *indx, double b[]);

void matinvert(double **mat, int n)
{
  double  **ident;
  int     *indx;
  int     i, j;
  double  d, norm;
    
  indx = malloc_1imatrix(n);
  ident = malloc_2dmatrix(n, n);

  for (i = 1; i <= n; i++) {
    for (j = 1; j <= n; j++) 
      ident[i][j] = 0.;
    ident[i][i] = 1.;
  }

  /* divide by mean eigenvalue of the matrix to stabilise the inversion */
  norm = 0.;
  for (i = 1; i <= n; i++)
    norm += mat[i][i];
  norm /= (double)n;

  for (i = 1; i <= n; i++)
    for (j = 1; j <= n; j++)
      mat[i][j] /= norm;

  /* perform LU decomposition and backsubtitution to invert matrix */
  matinv_ludcmp(mat, n, indx, &d);

  for (j = 1; j <= n; j++)
    matinv_lubksb(mat, n, indx, ident[j]);

  /* normalise the inverse */
  for (j = 1; j <= n; j++) 
    for (i = 1; i <= n; i++)
      mat[i][j] = ident[j][i] / norm;

      
  free_1imatrix(indx);
  free_2dmatrix(ident);

  return;
}

#define TINY 1.0e-20;

void matinv_ludcmp(double **a, int n, int *indx, double *d)
{
	int i,imax,j,k;
	double big,dum,sum,temp;
	double *vv;

	imax = 0;
	vv=malloc_1dmatrix(n);
	*d=1.0;
	for (i=1;i<=n;i++) {
		big=0.0;
		for (j=1;j<=n;j++)
			if ((temp=fabs(a[i][j])) > big) big=temp;
		if (big == 0.0) myerror("Singular matrix in routine ludcmp");
		vv[i]=1.0/big;
	}
	for (j=1;j<=n;j++) {
		for (i=1;i<j;i++) {
			sum=a[i][j];
			for (k=1;k<i;k++) sum -= a[i][k]*a[k][j];
			a[i][j]=sum;
		}
		big=0.0;
		for (i=j;i<=n;i++) {
			sum=a[i][j];
			for (k=1;k<j;k++)
				sum -= a[i][k]*a[k][j];
			a[i][j]=sum;
			if ( (dum=vv[i]*fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if (j != imax) {
			for (k=1;k<=n;k++) {
				dum=a[imax][k];
				a[imax][k]=a[j][k];
				a[j][k]=dum;
			}
			*d = -(*d);
			vv[imax]=vv[j];
		}
		indx[j]=imax;
		if (a[j][j] == 0.0) a[j][j]=TINY;
		if (j != n) {
			dum=1.0/(a[j][j]);
			for (i=j+1;i<=n;i++) a[i][j] *= dum;
		}
	}
	free_1dmatrix(vv);
}
#undef TINY

void matinv_lubksb(double **a, int n, int *indx, double b[])
{
	int i,ii=0,ip,j;
	double sum;

	for (i=1;i<=n;i++) {
		ip=indx[i];
		sum=b[ip];
		b[ip]=b[i];
		if (ii)
			for (j=ii;j<=i-1;j++) sum -= a[i][j]*b[j];
		else if (sum) ii=i;
		b[i]=sum;
	}
	for (i=n;i>=1;i--) {
		sum=b[i];
		for (j=i+1;j<=n;j++) sum -= a[i][j]*b[j];
		b[i]=sum/a[i][i];
	}
}

