/*  my_malloc.c                                                */
/*  Matrix Allocation routines                                 */
/*                                                             */
/*  KSM, 24.7.1997                                             */

/*  Time-stamp: <18 Oct 2002 18:17 BST my_malloc2.c maising> */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define MAXMATDIM 6
static int numdmalloc[MAXMATDIM+1], numdfree[MAXMATDIM+1];
static int numfmalloc[MAXMATDIM+1], numffree[MAXMATDIM+1];
static int numimalloc[MAXMATDIM+1], numifree[MAXMATDIM+1];
static int numgmalloc[MAXMATDIM+1], numgfree[MAXMATDIM+1];

void init_mymalloc()
{
  int i;
  for (i = 1; i <= MAXMATDIM; i++) {
    numdmalloc[i] = numdfree[i] = 0;
    numfmalloc[i] = numffree[i] = 0;
    numimalloc[i] = numifree[i] = 0;
    numgmalloc[i] = numgfree[i] = 0;
  }
  return;
}

void done_mymalloc()
{
  int i, flag;
  flag = 0;
  for (i = 1; i <= MAXMATDIM; i++) 
    if ((numdmalloc[i] - numdfree[i] != 0) || 
	(numfmalloc[i] - numffree[i] != 0) ||
	(numimalloc[i] - numifree[i] != 0) ||
        (numgmalloc[i] - numgfree[i] != 0))
      flag = 1;
  if (flag) {
    printf("Not enough memory freed:\n");
    for (i = 1; i <= MAXMATDIM; i++) {
      if (numfmalloc[i] - numffree[i] != 0)  
	  printf("malloc_%if: %i %i %i calls\n", i, numfmalloc[i],  
		 numffree[i], numfmalloc[i]-numffree[i]); 
      if (numdmalloc[i] - numdfree[i] != 0)  
	printf("malloc_%id: %i %i %i calls\n", i, numdmalloc[i],  
	       numdfree[i], numdmalloc[i]-numdfree[i]); 
      if (numimalloc[i] - numifree[i] != 0)  
	printf("malloc_%ii: %i %i %i calls\n", i, numimalloc[i],  
	       numifree[i], numimalloc[i]-numifree[i]); 
      if (numgmalloc[i] - numgfree[i] != 0)  
	printf("malloc_%ig: %i %i %i calls\n", i, numgmalloc[i],  
	       numgfree[i], numgmalloc[i]-numgfree[i]); 
    }
  }
  return;
}

void reg_dmalloc(int num)
{
  numdmalloc[num]++;
  return;
}

void reg_imalloc(int num)
{
  numimalloc[num]++;
  return;
}

void reg_fmalloc(int num)
{
  numfmalloc[num]++;
  return;
}

void reg_malloc(int num)
{
  numgmalloc[num]++;
  return;
}

void myerror(char error_text[])
/* Numerical Recipes standard error handler */
{
        fprintf(stderr,"Run-time error...\n");
        fprintf(stderr,"%s\n",error_text);
        fprintf(stderr,"...now exiting to system...\n");
        exit(1);
}

void warning(char error_text[])
{
  fprintf(stdout, "Warning: %s\n", error_text);
  return;
} 

int write_file(char filename[], char new_text[])
{
  FILE *fp;

  fp = fopen(filename, "a");
  if (fp) {
    fprintf(fp, "%s", new_text);
    fflush(fp);
    fclose(fp);
    return 0;
  }
  return -1;
}

int debug(char filename[], char error_text[])
{
  FILE *fdebug;
  time_t tp;

  fdebug = fopen(filename, "a");
  if (fdebug) {
    time(& tp);
    fprintf(fdebug, "%s-%s\n", ctime(&tp), error_text);
    fflush(fdebug);
    fclose(fdebug);
    return 0;
  }
  return -1;
}

void print_1imatrix(int *t, int n1) 
{
  int k;

  printf("(");
  for (k=1; k <= n1; k++) 
    printf("%i ", t[k]);
  printf(")\n");

  return;
}

void print_1fmatrix(float *t, int n1) 
{
  int k;

  printf("(");
  for (k=1; k <= n1; k++) 
    printf("%f ", t[k]);
  printf(")\n");

  return;
}

void print_2fmatrix(float **t, int n1, int n2) 
{
  int i, k;

  for (i=1; i <= n1; i++) {
    printf("(");
    for (k=1; k <= n2; k++) 
      printf("%g ", t[i][k]);
    printf(")\n");
  }
  return;
}

void print_1dmatrix(double *t, int n1) 
{
  int k;

  printf("(");
  for (k=1; k <= n1; k++) 
    printf("%g ", t[k]);
  printf(")\n");

  return;
}

void print_2dmatrix(double **t, int n1, int n2) 
{
  int i, k;

  for (i=1; i <= n1; i++) {
    printf("(");
    for (k=1; k <= n2; k++) 
      printf("%g ", t[i][k]);
    printf(")\n");
  }
  return;
}

void print_3dmatrix(double ***t, int n1, int n2, int n3) 
{
  int j, k, l;

  for (j=1; j <= n1; j++) {
    printf("(");
    for (k=1; k <= n2; k++) 
      for (l=1; l <= n3; l++) 
	printf("%g ", t[j][k][l]);
    printf(")\n");
  }
  return;
}

void print_4dmatrix(double ****t, int n1, int n2, int n3, int n4) 
{
  int i, j, k, l;

  for (i=1; i <= n1; i++) 
    for (j=1; j <= n2; j++) {
      printf("(");
      for (k=1; k <= n3; k++) 
	for (l=1; l <= n4; l++) 
          printf("%g ", t[i][j][k][l]);
      printf(")\n");
    }
  return;
}

void print_5dmatrix(double *****t, int n1, int n2, int n3, int n4, int n5) 
{
  int i, j, k, l, m;

  for (i=1; i <= n1; i++) 
    for (j=1; j <= n2; j++) 
      for (k=1; k <= n3; k++) { 
        printf("(");
	for (l=1; l <= n4; l++) 
	  for (m=1; m <= n5; m++) 
	    printf("%g ", t[i][j][k][l][m]);
	printf(")\n");
      }
  return;
}

void print_6dmatrix(double ******t, int n1, int n2, int n3, int n4, int n5, int n6) 
{
  int i, j, k, l, m, n;

  for (i=1; i <= n1; i++) 
    for (j=1; j <= n2; j++) 
      for (k=1; k <= n3; k++) { 
	printf("(");
	for (l=1; l <= n4; l++) 
	  for (m=1; m <= n5; m++)
	    for (n=1; n <= n6; n++)
	      printf("%g ", t[i][j][k][l][m][n]);
	printf(")\n");
      }
  return;
}

/***********************************************************************/
int *malloc_1imatrix(int n)
{
  int *m;
  char errormsg[80];
  numimalloc[1]++;
  m = (int *) malloc((size_t)(n*sizeof(int))) - 1;
  if (!(m+1)) {
    sprintf(errormsg, "allocation failure 1 in malloc_1imatrix(%i)", n);
    myerror(errormsg);
  }
  return m;
}

int **malloc_2imatrix(int n1, int n2)
{
  int i;
  int **m;

  numimalloc[2]++;
  m = (int **) malloc((size_t)(n1*sizeof(int *))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_2imatrix()");
  m[1] = (int *) malloc((size_t)(n1*n2*sizeof(int))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_2imatrix()");

  for (i = 1 + 1; i <= n1; i++)
    m[i] = m[i-1] + n2;

  return m;
}

void free_1imatrix(int *m)
{
  numifree[1]++;
  free( (char*) (m+1));
  return;
}

void free_2imatrix(int **m)
{
  numifree[2]++;
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}

/***********************************************************************/
float *malloc_1fmatrix(int n)
{
  float *m;
  numfmalloc[1]++;
  m = (float *) malloc((size_t)(n*sizeof(float))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_1fmatrix()");
  return m;
}

void free_1fmatrix(float *m)
{
  numffree[1]++;
  free( (char*) (m+1));
  return;
}

float **malloc_2fmatrix(int n1, int n2)
{
  int i;
  float **m;

  numfmalloc[2]++;
  m = (float **) malloc((size_t)(n1*sizeof(float *))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_2fmatrix()");
  m[1] = (float *) malloc((size_t)(n1*n2*sizeof(float))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_2fmatrix()");

  for (i = 1 + 1; i <= n1; i++)
    m[i] = m[i-1] + n2;

  return m;
}

float ***malloc_3fmatrix(int n1, int n2, int n3)
{
  int i, j;
  float ***m;

  numfmalloc[3]++;
  m = (float ***) malloc((size_t)(n1*sizeof(float**))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_3fmatrix()");
  m[1] = (float **) malloc((size_t)(n1*n2*sizeof(float*))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_3fmatrix()");
  m[1][1] = (float *) malloc((size_t)(n1*n2*n3*sizeof(float))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_3fmatrix()");

  for(i = 1 + 1; i <= n1; i++) {
    m[i] =    m[i-1]+n2;
    m[i][1] = m[i-1][1]+n2*n3;
  }
  for(i = 1; i <= n1; i++) 
    for(j = 1 + 1; j <= n2; j++) m[i][j] = m[i][j-1]+n3;

  return m;
}

float ****malloc_4fmatrix(int n1, int n2, int n3, int n4)
{
  int i, j, h;
  float ****m;
  char errormsg[80], argstr[80];

  sprintf(argstr, "(%i, %i, %i, %i)", n1, n2, n3, n4);
  numfmalloc[4]++;
  m = (float ****) malloc((size_t)(n1*sizeof(float***))) - 1;
  if (!(m+1)) {
    sprintf(errormsg, "allocation failure 1 in malloc_4fmatrix%s",
	    argstr);
    myerror(errormsg);
  }
  m[1] = (float ***) malloc((size_t)(n1*n2*sizeof(float**))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_4fmatrix()");
  m[1][1] = (float **) malloc((size_t)(n1*n2*n3*sizeof(float*))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_4fmatrix()");
  m[1][1][1] = (float *) malloc((size_t)(n1*n2*n3*n4*sizeof(float))) - 1;
  if (!(m[1][1][1]+1)) myerror("allocation failure 4 in malloc_4fmatrix()");

  for (i = 1 + 1; i <= n1; i++) {
    m[i] =          m[i-1]+n2;
    m[i][1] =       m[i-1][1]+n2*n3;
    m[i][1][1] =    m[i-1][1][1]+n2*n3*n4;
  }
  for (i = 1; i <= n1; i++) {
    for (j = 1 + 1; j <= n2; j++) {
      m[i][j] =       m[i][j-1]+n3;
      m[i][j][1] =    m[i][j-1][1]+n3*n4;
    }
    for (j = 1 ; j <= n2; j++) {
      for(h = 1 + 1; h <= n3; h++)
	m[i][j][h] =    m[i][j][h-1]+n4;
    }
  }

  return m;
}

void free_2fmatrix(float **m)
{
  numffree[2]++;
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}

void free_3fmatrix(float ***m)
{
  numffree[3]++;
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}

void free_4fmatrix(float ****m)
{
  numffree[4]++;
  free( (char*) (m[1][1][1]+1));
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;

}

/***********************************************************************/
void *malloc_1matrix(int n, size_t size)
{
  char *m;
  numgmalloc[1]++;
  m = (char *) malloc((size_t)(n*size)) - size;
  if (!(m+size)) myerror("allocation failure 1 in malloc_1matrix()");
  return (void *)m;
}

void **malloc_2matrix(int n1, int n2, size_t size)
{
  int i;
  char **m;

  numgmalloc[2]++;
  m = (char **)malloc((size_t)(n1*sizeof(char *))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_2matrix()");
  m[1] = (char *)malloc((size_t)(n1*n2*size))-size;
  if (!(m[1]+size)) myerror("allocation failure 2 in malloc_2matrix()");

  for (i = 1 + 1; i <= n1; i++)
    m[i] = m[i-1] + n2*size;

  return (void **)m;
}

void ***malloc_3matrix(int n1, int n2, int n3, size_t size)
{
  int i, j;
  char ***m;

  numgmalloc[3]++;
  m = (char ***) malloc((size_t)(n1*sizeof(char**))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_3matrix()");
  m[1] = (char **) malloc((size_t)(n1*n2*sizeof(char*))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_3matrix()");
  m[1][1] = (char *) malloc((size_t)(n1*n2*n3*size)) - size;
  if (!(m[1][1]+size)) myerror("allocation failure 3 in malloc_3matrix()");

  for(i = 1 + 1; i <= n1; i++) {
    m[i] =    m[i-1]+n2;
    m[i][1] = m[i-1][1]+n2*(n3*size);
  }
  for(i = 1; i <= n1; i++) 
    for(j = 1 + 1; j <= n2; j++) m[i][j] = m[i][j-1]+(n3*size);

  return (void ***)m;
}

void ****malloc_4matrix(int n1, int n2, int n3, int n4, size_t size)
{
  int i, j, h;
  char ****m;

  numgmalloc[4]++;
  m = (char ****) malloc((size_t)(n1*sizeof(char***))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_4matrix()");
  m[1] = (char ***) malloc((size_t)(n1*n2*sizeof(char**))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_4matrix()");
  m[1][1] = (char **) malloc((size_t)(n1*n2*n3*sizeof(char*))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_4matrix()");
  m[1][1][1] = (char *) malloc((size_t)(n1*n2*n3*n4*size)) - size;
  if (!(m[1][1][1]+size)) myerror("allocation failure 4 in malloc_4matrix()");

  for (i = 1 + 1; i <= n1; i++) {
    m[i] =          m[i-1]+n2;
    m[i][1] =       m[i-1][1]+n2*n3;
    m[i][1][1] =    m[i-1][1][1]+n2*n3*(n4*size);
  }
  for (i = 1; i <= n1; i++) {
    for (j = 1 + 1; j <= n2; j++) {
      m[i][j] =       m[i][j-1]+n3;
      m[i][j][1] =    m[i][j-1][1]+n3*(n4*size);
    }
    for (j = 1 ; j <= n2; j++) {
      for(h = 1 + 1; h <= n3; h++)
	m[i][j][h] =    m[i][j][h-1]+(n4*size);
    }
  }
  return (void ****)m;
}

void free_1matrix(void *m, size_t size)
{
  numgfree[1]++;
  free( ((char*) m)+size);
  return;
}

void free_2matrix(void **m, size_t size)
{
  numgfree[2]++;
  free( ((char*) (m[1]))+size);
  free( (char*) (m+1));
  return;
}

void free_3matrix(void ***m, size_t size)
{
  numgfree[3]++;
  free( ((char*) m[1][1])+size);
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}

void free_4matrix(void ****m, size_t size)
{
  numgfree[4]++;
  free( ((char*) m[1][1][1])+size);
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;

}
/***********************************************************************/

double *malloc_1dmatrix(int n)
{
  double *m;
  numdmalloc[1]++;
  m = (double *) malloc((size_t)(n*sizeof(double))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_1dmatrix()");
  return m;
}

void free_1dmatrix(double *m)
{
  numdfree[1]++;
  free( (char*) (m+1));
  return;
}

double **malloc_2dmatrix(int n1, int n2)
{
  int i;
  double **m;

  numdmalloc[2]++;
  m = (double **) malloc((size_t)(n1*sizeof(double *))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_2dmatrix()");
  m[1] = (double *) malloc((size_t)(n1*n2*sizeof(double))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_2dmatrix()");

  for (i = 1 + 1; i <= n1; i++)
    m[i] = m[i-1] + n2;

  return m;
}

double ***malloc_3dmatrix(int n1, int n2, int n3)
{
  int i, j;
  double ***m;

  numdmalloc[3]++;
  m = (double ***) malloc((size_t)(n1*sizeof(double**))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_3dmatrix()");
  m[1] = (double **) malloc((size_t)(n1*n2*sizeof(double*))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_3dmatrix()");
  m[1][1] = (double *) malloc((size_t)(n1*n2*n3*sizeof(double))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_3dmatrix()");

  for(i = 1 + 1; i <= n1; i++) {
    m[i] =    m[i-1]+n2;
    m[i][1] = m[i-1][1]+n2*n3;
  }
  for(i = 1; i <= n1; i++) 
    for(j = 1 + 1; j <= n2; j++) m[i][j] = m[i][j-1]+n3;

  return m;
}

double ****malloc_4dmatrix(int n1, int n2, int n3, int n4)
{
  int i, j, h;
  double ****m;
  char errormsg[80], argstr[80];

  sprintf(argstr, "(%i, %i, %i, %i)", n1, n2, n3, n4);
  numdmalloc[4]++;
  m = (double ****) malloc((size_t)(n1*sizeof(double***))) - 1;
  if (!(m+1)) {
    sprintf(errormsg, "allocation failure 1 in malloc_4dmatrix%s",
	    argstr);
    myerror(errormsg);
  }
  m[1] = (double ***) malloc((size_t)(n1*n2*sizeof(double**))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_4dmatrix()");
  m[1][1] = (double **) malloc((size_t)(n1*n2*n3*sizeof(double*))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_4dmatrix()");
  m[1][1][1] = (double *) malloc((size_t)(n1*n2*n3*n4*sizeof(double))) - 1;
  if (!(m[1][1][1]+1)) myerror("allocation failure 4 in malloc_4dmatrix()");

  for (i = 1 + 1; i <= n1; i++) {
    m[i] =          m[i-1]+n2;
    m[i][1] =       m[i-1][1]+n2*n3;
    m[i][1][1] =    m[i-1][1][1]+n2*n3*n4;
  }
  for (i = 1; i <= n1; i++) {
    for (j = 1 + 1; j <= n2; j++) {
      m[i][j] =       m[i][j-1]+n3;
      m[i][j][1] =    m[i][j-1][1]+n3*n4;
    }
    for (j = 1 ; j <= n2; j++) {
      for(h = 1 + 1; h <= n3; h++)
	m[i][j][h] =    m[i][j][h-1]+n4;
    }
  }

  return m;
}

double *****malloc_5dmatrix(int s1, int s2, int s3, int n1, int n2)
{
  int i,j,k,l;

  double *****m;
  
  numdmalloc[5]++;
  m = (double *****)malloc((size_t)((s1)*sizeof(double ****))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_5dmatrix()");
  m[1] = (double ****)malloc((size_t)((s1*s2)*sizeof(double ***))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_5dmatrix()");
  m[1][1] = (double ***)malloc((size_t)((s1*s2*s3)*sizeof(double **))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_5dmatrix()");
  m[1][1][1] = (double **)malloc((size_t)((s1*s2*s3*n1)
					    *sizeof(double *))) - 1;
  if (!(m[1][1][1]+1)) myerror("allocation failure 4 in malloc_5dmatrix()");
  m[1][1][1][1] = (double *)malloc((size_t)((s1*s2*s3*n1*n2)
					    *sizeof(double))) - 1;
  if (!(m[1][1][1][1]+1)) 
    myerror("allocation failure 5 in malloc_5dmatrix()");
  
  for (i = 1 + 1; i <= s1; i++) {
    m[i] =          m[i-1]+s2;
    m[i][1] =       m[i-1][1]+s2*s3;
    m[i][1][1] =    m[i-1][1][1]+s2*s3*n1;
    m[i][1][1][1] = m[i-1][1][1][1]+s2*s3*n1*n2;
  }
  for (i = 1; i <= s1; i++) {
    for (j = 1 + 1; j <= s2; j++) {
      m[i][j] =       m[i][j-1]+s3;
      m[i][j][1] =    m[i][j-1][1]+s3*n1;
      m[i][j][1][1] = m[i][j-1][1][1]+s3*n1*n2;
    }
    for (j = 1 ; j <= s2; j++) {
      for(k = 1 + 1; k <= s3; k++) {
	m[i][j][k] =    m[i][j][k-1]+n1;
	m[i][j][k][1] = m[i][j][k-1][1]+n1*n2;
      }
      for(k = 1; k <= s3; k++) 
	for(l = 1 + 1; l <= n1; l++) 
	  m[i][j][k][l] = m[i][j][k][l-1]+n2;
    }
  }

  return m;
  
}
     
double ******malloc_6dmatrix(int s0, int s1, int s2, int s3, int n1, int n2)
{
  int i,j,k,l,n;

  double ******m;
  
  numdmalloc[6]++;
  m = (double ******)malloc((size_t)((s0)*sizeof(double *****))) - 1;
  if (!(m+1)) myerror("allocation failure 1 in malloc_6dmatrix()");
  m[1] = (double *****)malloc((size_t)((s0*s1)*sizeof(double ****))) - 1;
  if (!(m[1]+1)) myerror("allocation failure 2 in malloc_6dmatrix()");
  m[1][1] = (double ****)malloc((size_t)((s0*s1*s2)*sizeof(double ***))) - 1;
  if (!(m[1][1]+1)) myerror("allocation failure 3 in malloc_6dmatrix()");
  m[1][1][1] = (double ***)malloc((size_t)((s0*s1*s2*s3)
					    *sizeof(double **))) - 1;
  if (!(m[1][1][1]+1)) myerror("allocation failure 4 in malloc_6dmatrix()");

  m[1][1][1][1] = (double **)malloc((size_t)((s0*s1*s2*s3*n1)
					    *sizeof(double *))) - 1;
  if (!(m[1][1][1][1]+1)) 
    myerror("allocation failure 5 in malloc_6dmatrix()");
  m[1][1][1][1][1] = (double *)malloc((size_t)((s0*s1*s2*s3*n1*n2)
					    *sizeof(double))) - 1;
  if (!(m[1][1][1][1][1]+1)) 
    myerror("allocation failure 6 in malloc_6dmatrix()");

  
  for (i = 1 + 1; i <= s0; i++) {
    m[i] =             m[i-1]+s1;
    m[i][1] =          m[i-1][1]+s1*s2;
    m[i][1][1] =       m[i-1][1][1]+s1*s2*s3;
    m[i][1][1][1] =    m[i-1][1][1][1]+s1*s2*s3*n1;
    m[i][1][1][1][1] = m[i-1][1][1][1][1]+s1*s2*s3*n1*n2;
  }
  for (i = 1; i <= s0; i++) {
    for (j = 1 + 1; j <= s1; j++) {
      m[i][j] =          m[i][j-1]+s2;
      m[i][j][1] =       m[i][j-1][1]+s2*s3;
      m[i][j][1][1] =    m[i][j-1][1][1]+s2*s3*n1;
      m[i][j][1][1][1] = m[i][j-1][1][1][1]+s2*s3*n1*n2;
    }
    for (j = 1 ; j <= s1; j++) {
      for(k = 1 + 1; k <= s2; k++) {
	m[i][j][k] =       m[i][j][k-1]+s3;
	m[i][j][k][1] =    m[i][j][k-1][1]+s3*n1;
	m[i][j][k][1][1] = m[i][j][k-1][1][1]+s3*n1*n2;
      }
      for(k = 1; k <= s2; k++) {
	for(l = 1 + 1; l <= s3; l++) { 
	  m[i][j][k][l] =    m[i][j][k][l-1]+n1;
	  m[i][j][k][l][1] = m[i][j][k][l-1][1]+n1*n2;
	}
	for(l = 1; l <= s3; l++) {
	  for (n = 1 + 1; n <= n1; n++)
	    m[i][j][k][l][n] = m[i][j][k][l][n-1]+n2;
	}
      }
    }
  }

#if 1
  {
    double *m2;
    m2 = m[1][1][1][1][1];
    for(i=1; i <= s0*s1*s2*s3*n1*n2; i++)
      *(m2+i) = 0.;
  }
#endif
  return m;
  
}
     
void free_2dmatrix(double **m)
{
  numdfree[2]++;
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}

void free_3dmatrix(double ***m)
{
  numdfree[3]++;
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}

void free_4dmatrix(double ****m)
{
  numdfree[4]++;
  free( (char*) (m[1][1][1]+1));
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;

}

void free_5dmatrix(double *****m)
{
  numdfree[5]++;
  free( (char*) (m[1][1][1][1]+1));
  free( (char*) (m[1][1][1]+1));
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;

}

void free_6dmatrix(double ******m)
{
  numdfree[6]++;
  free( (char*) (m[1][1][1][1][1]+1));
  free( (char*) (m[1][1][1][1]+1));
  free( (char*) (m[1][1][1]+1));
  free( (char*) (m[1][1]+1));
  free( (char*) (m[1]+1));
  free( (char*) (m+1));
  return;
}
