/*
 * some astronomical conversion routines, partly based on SLALIB 
 * (see http://star-www.rl.ac.uk/) 
 * KSM, 2001
 */
/*  Time-stamp: <24 Jan 2002 12:38 GMT myslalib.c maising> */

#include <math.h>
#include "myslalib.h"

#ifndef PI
#define PI 3.14159265358979323844
#endif

/* 
 * SLALIB routine:
 * Convert hours, minutes, seconds to radians 
 */
void sla_ctf2r(double hour, double min, double sec, double *rad, int *status)
{
  int ihour, imin;
  const double d2s = 86400.;

  ihour = (int)hour;
  imin =  (int)min;

  *status = 0;
  if ((sec < 0.)  || (sec >= 60.)) *status = 3;
  if ((imin < 0)  || (imin > 59))  *status = 2;
  if ((ihour < 0) || (ihour > 23)) *status = 1;
  *rad = 2. * PI * (60. * (60. * ihour + imin) + sec) / d2s;

  return;
}

/* 
 * SLALIB routine:
 * Convert degrees, arcminutes, arcseconds to radians
 */
void sla_daf2r(double deg, double amin, double asec, double *rad, int *status)
{
  int ideg, iamin;
  const double as2r = 4.84813681109535994e-6;

  ideg  = (int)deg;
  iamin =  (int)amin;

  *status = 0;
  if ((asec < 0.)  || (asec >= 60.)) *status = 3;
  if ((iamin < 0)  || (iamin > 59))  *status = 2;
  if ((ideg < 0) || (deg > 23)) *status = 1;
  *rad = as2r * (60. * (60. * ideg + iamin) + asec);

  return;
}

/* 
 *  Convert an angle in radians into degrees, arcminutes, arcseconds
 */
void rad2deg(double rad, 
	     int *sign, double *deg, double *arcmin, double *arcsec)
{
  const double r2d = 180./PI;

  if (rad >= 0.)
    *sign = 1;
  else 
    *sign = -1;
  rad *= r2d;
  rad = fabs(rad);
  *deg = floor(rad);
  rad -= *deg;
  rad *= 60.;
  *arcmin = floor(rad);
  rad -= *arcmin;
  *arcsec = rad * 60.;

  return;
}

void rad2hr(double rad,
	    int *sign, double *hour, double *min, double *sec)
{
  rad2deg(rad/15., sign, hour, min, sec);

  return;
}
