c
C     evaluate quadratic form and determinant using SLATEC routines
C     HISTORY
C        ??-??-97 (??)  MPH  original version
C        ??-??-01       KSM  some bugfixing to make it work with C
C
 
c      Time-stamp: <28 Jan 2002 20:34 GMT spquadform.f maising>

c     |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c     NOTE: Some of the SLATEC routines have misleading header
c           comments that DO NOT CORRECTLY describe the dimensions
c           of the required workspace arrays!!!!
c     |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c     This implementation hopefully works after comparison
c     with the actual SLATEC source.
c     But don't call dchkw - it doesn't know what it's doing. :-(
c     perhaps I should submit a bug report some day...

c-----*-----------------------------------------------------------------
c     this subroutine performs an incomplete Cholesky decomposition 
c     of a matrix in compressed column storage. 
c     arguments: n - dimension of matrix
c                nz - number of non-zero elements
c                rwork - workspace array dimension nz+5*n
c                iwork - workspace array dimension nz+n+11
c                x - workspace array dimension n
c
      subroutine spquadform(n,nz,ia,ja,sa,b,quad,lndet,quaderr, 
     &     iwork, rwork, x)
      implicit none
      integer  n,nz
      integer  ia(nz),ja(nz)
      real*8   sa(nz)
      real*8   b(n)
      real*8   quad,lndet,quaderr
      integer  iwork(nz+n+11)
      real*8   rwork(nz+5*n)      
      real*8   x(n)

      integer  lenw, leniw
      integer  isym,itol,itmax,iter,ierr,iunit
      real*8   tol,err

      integer  i
      integer  locrb,locib
      parameter (locrb=1,locib=11)
      integer  locdin,locdz,locel,lociel,lociw,locjel,locp,locr
      integer  locw,locz,nl

      external dcg,dchkw,ds2y,dsics,dsllti,dsmv,xermsg

c initialise variables

      isym=1
      itol=1
      tol=1.0d-5
      itmax=100
      iunit=0

c count number of elements in lower triangle of the matrix

      if (isym.eq.0) then
        nl=(nz+n)/2
      else
        nl=nz
      end if

c set up the work arrays

      lenw=nz+5*n 
      leniw=nz+n+11

      locjel=locib
      lociel=locjel+nl
      lociw=lociel+n+1
      locel=locrb
      locdin=locel+nl
      locr=locdin+n
      locz=locr+n
      locp=locz+n
      locdz=locp+n
      locw=locdz+n

c check the workspace allocations

c      call dchkw('dsiccg',lociw,leniw,locw,lenw,ierr,iter,err)
c      if (ierr.ne.0) write(*,*) 'WARNING: dchwk - ierr = ',ierr
      iwork(1)=nl
      iwork(2)=locjel
      iwork(3)=lociel
      iwork(4)=locel
      iwork(5)=locdin
      iwork(9)=lociw
      iwork(10)=locw

c compute the incomplete Cholesky decomposition of sa

      call dsics(n,nz,ia,ja,sa,isym,nl,iwork(lociel),iwork(locjel),
     &           rwork(locel),rwork(locdin),rwork(locr),ierr)
      if (ierr.ne.0) write(*,*) 'WARNING: dsics - ierr = ',ierr

c calculate determinant of sparse matrix sa

      lndet=0d0
      do i=1,n
        lndet=lndet-dlog(rwork(locdin-1+i))
      end do

c     initialise x; failure to do so can cause nasty problems
      do i=1,n
         x(i) = 0.
      end do

c solve (sa)x=b by conjugate gradient algorithm

      call dcg(n,b,x,nz,ia,ja,sa,isym,dsmv,dsllti,itol,tol,itmax,
     &         iter,err,ierr,iunit,rwork(locr),rwork(locz),rwork(locp),
     &         rwork(locdz),rwork(1),iwork(1))
      if (ierr.ne.0) write(*,*) 'WARNING: dcg - ierr = ',ierr

      quad=0d0
      quaderr=0d0
      do i=1,n
        quad=quad+b(i)*x(i)
        quaderr=quaderr+(b(i)*err)**2
      end do
      quaderr=dsqrt(quaderr)

      return
      end

c-----*-----------------------------------------------------------------
